@@ -1,7 +1,7 @@
 /*
  * bibtypes.c
  *
- * Copyright (c) Chris Putnam 2003-8
+ * Copyright (c) Chris Putnam 2003-2009
  *
  * Program and source code released under the GPL
  *
@@ -38,6 +38,9 @@ static lookups article[] = {
 	{ "location",     "LOCATION",  SIMPLE, LEVEL_MAIN },
 	{ "ftp",          "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "url",          "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "howpublished", "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "doi",          "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "key",          "BIBKEY",       SIMPLE, LEVEL_MAIN },
@@ -80,6 +83,9 @@ static lookups book[] = {
 	{ "key",          "BIBKEY",    SIMPLE, LEVEL_MAIN },
 	{ "ftp",          "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "url",          "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "address"       "ADDRESS",   SIMPLE, LEVEL_MAIN },
 	{ "howpublished", "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "doi",          "DOI",       SIMPLE, LEVEL_MAIN },
@@ -119,6 +125,9 @@ static lookups report[] = {
 	{ "doi",          "DOI",          SIMPLE, LEVEL_MAIN },
 	{ "ftp",          "URL",          BIBTEX_URL, LEVEL_MAIN },
 	{ "url",          "URL",          BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "howpublished", "URL",          BIBTEX_URL, LEVEL_MAIN },
 	{ "refnum",    "REFNUM",          SIMPLE, LEVEL_MAIN },
 	{ "crossref",     "CROSSREF",  SIMPLE, LEVEL_MAIN },
@@ -152,6 +161,9 @@ static lookups manual[] = {
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "howpublished", "URL",    BIBTEX_URL, LEVEL_MAIN },
 	{ "language",     "LANGUAGE",  SIMPLE, LEVEL_MAIN },
 	{ "refnum",    "REFNUM",    SIMPLE, LEVEL_MAIN },
@@ -190,6 +202,9 @@ static lookups inbook[] = {
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "howpublished", "URL",    BIBTEX_URL, LEVEL_MAIN },
 	{ "refnum",    "REFNUM",    SIMPLE, LEVEL_MAIN },
 	{ "crossref",     "CROSSREF",  SIMPLE, LEVEL_MAIN },
@@ -231,6 +246,9 @@ static lookups inproceedings[] = {
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "type",      "TYPE",      SIMPLE, LEVEL_MAIN },
 	{ "note",         "NOTES",        SIMPLE, LEVEL_MAIN },
 	{ "key",          "BIBKEY",          SIMPLE, LEVEL_MAIN },
@@ -273,6 +291,9 @@ static lookups incollection[] = {
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "location",     "LOCATION",     SIMPLE, LEVEL_HOST },
 	{ "howpublished", "URL",    BIBTEX_URL, LEVEL_MAIN },
 	{ "refnum",    "REFNUM",    SIMPLE, LEVEL_MAIN },
@@ -309,6 +330,9 @@ static lookups proceedings[] = {
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "language",     "LANGUAGE",     SIMPLE, LEVEL_MAIN },
 	{ "howpublished", "URL",    BIBTEX_URL, LEVEL_MAIN },
 	{ "location",     "LOCATION",     SIMPLE, LEVEL_MAIN },
@@ -336,6 +360,9 @@ static lookups phds[] = {
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "howpublished", "URL" ,   BIBTEX_URL, LEVEL_MAIN },
 	{ "language",     "LANGUAGE",     SIMPLE, LEVEL_MAIN },
 	{ "location",     "LOCATION",     SIMPLE, LEVEL_MAIN },
@@ -365,6 +392,9 @@ static lookups masters[] = {
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "language",     "LANGUAGE",     SIMPLE, LEVEL_MAIN },
 	{ "location",     "LOCATION",     SIMPLE, LEVEL_MAIN },
 	{ "note",         "NOTES",        SIMPLE, LEVEL_MAIN },
@@ -398,6 +428,9 @@ static lookups unpublished[] = {
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "language",     "LANGUAGE",     SIMPLE, LEVEL_MAIN },
 	{ "howpublished", "URL",    BIBTEX_URL, LEVEL_MAIN },
 	{ "location",     "LOCATION",     SIMPLE, LEVEL_MAIN },
@@ -434,6 +467,9 @@ static lookups standard[] = {
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "howpublished", "URL",    BIBTEX_URL, LEVEL_MAIN },
 	{ "language",     "LANGUAGE",     SIMPLE, LEVEL_MAIN },
 	{ "refnum",    "REFNUM",    SIMPLE, LEVEL_MAIN },
@@ -470,6 +506,9 @@ static lookups periodical[] = {
 	{ "ftp",          "URL",         BIBTEX_URL, LEVEL_MAIN },
 	{ "doi",          "DOI",         SIMPLE, LEVEL_MAIN },
 	{ "url",          "URL",         BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "howpublished", "URL",         BIBTEX_URL, LEVEL_MAIN },
 	{ "refnum",       "REFNUM",      SIMPLE, LEVEL_MAIN },
 	{ "language",     "LANGUAGE",    SIMPLE, LEVEL_MAIN },
@@ -508,6 +547,9 @@ static lookups patent[] = {
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "howpublished", "URL",    BIBTEX_URL, LEVEL_MAIN },
 	{ "type",       "TYPE",     SIMPLE, LEVEL_MAIN },
 	{ "refnum",    "REFNUM",    SIMPLE, LEVEL_MAIN },
@@ -539,6 +581,9 @@ static lookups electronic[] = {
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "howpublished", "URL",    BIBTEX_URL, LEVEL_MAIN },
 	{ "refnum",    "REFNUM",    SIMPLE, LEVEL_MAIN },
 	{ "crossref",     "CROSSREF",  SIMPLE, LEVEL_MAIN },
@@ -569,6 +614,9 @@ static lookups misc[] = {
 	{ "ftp",       "URL",       BIBTEX_URL, LEVEL_MAIN },
 	{ "doi",       "DOI",       SIMPLE, LEVEL_MAIN },
 	{ "url",       "URL",       BIBTEX_URL, LEVEL_MAIN },
+	{ "pdf",          "FILEATTACH",SIMPLE, LEVEL_MAIN },
+	{ "sentelink",    "FILEATTACH",BIBTEX_SENTE, LEVEL_MAIN },
+	{ "file",         "FILEATTACH",BIBTEX_FILE,  LEVEL_MAIN },
 	{ "howpublished", "URL",    BIBTEX_URL, LEVEL_MAIN },
 	{ "refnum",    "REFNUM",    SIMPLE, LEVEL_MAIN },
 	{ "crossref",     "CROSSREF",  SIMPLE, LEVEL_MAIN },
