@@ -1,7 +1,7 @@
 /*
  * modsout.c
  *
- * Copyright (c) Chris Putnam 2003-8
+ * Copyright (c) Chris Putnam 2003-2009
  *
  * Source code released under the GPL
  *
@@ -40,16 +40,6 @@ output_tab1( FILE *outptr, int level, char *tag )
 	fprintf( outptr, "%s", tag );
 }
 
-#if 0
-static void
-output_tab2( FILE *outptr, int level, char *tag, char *data, int cr )
-{
-	output_tab0( outptr, level );
-	fprintf( outptr, "<%s>%s</%s>", tag, data, tag );
-	if ( cr ) fprintf( outptr, "\n" );
-}
-#endif
-
 static void
 output_tab2_attrib( FILE *outptr, int level, char *tag, char *data, 
 	char *attrib, char *type, int cr )
@@ -80,19 +70,6 @@ output_fill2( FILE *outptr, int level, char *tag, fields *info, int n, int cr )
 	}
 }
 
-#if 0
-static void
-output_fill2_attrib( FILE *outptr, int level, char *tag, char *attrib, char *type,
-	fields *info, int n, int cr )
-{
-	if ( n!=-1 ) {
-		output_tab2_attrib( outptr, level, tag, info->data[n].data, 
-			attrib, type, cr );
-		fields_setused( info, n );
-	}
-}
-#endif
-
 static void
 output_fill4( FILE *outptr, int level, char *tag, char *aname, char *avalue,
 		fields *info, int n, int cr )
@@ -104,6 +81,23 @@ output_fill4( FILE *outptr, int level, char *tag, char *aname, char *avalue,
 	}
 }
 
+/*
+ * Find the positions of all convert.internal tags and store the
+ * locations in convert.code.
+ *
+ * Return number of the tags found
+ */
+static int
+find_alltags( fields *info, convert *parts, int nparts, int level )
+{
+	int i, n=0;
+	for ( i=0; i<nparts; ++i ) {
+		parts[i].code = fields_find( info, parts[i].internal, level );
+		n += ( parts[i].code!=-1 );
+	}
+	return n;
+}
+
 static void
 output_title( fields *info, FILE *outptr, int level )
 {
@@ -145,8 +139,7 @@ output_name( FILE *outptr, char *p, int level )
 	newstr family, part;
 	int n=0;
 
-	newstr_init( &family );
-	newstr_init( &part );
+	newstrs_init( &family, &part, NULL );
 
 	while ( *p && *p!='|' ) newstr_addchar( &family, *p++ );
 	if ( *p=='|' ) p++;
@@ -176,8 +169,7 @@ output_name( FILE *outptr, char *p, int level )
 				family.data, 1 );
 	}
 
-	newstr_free( &part );
-	newstr_free( &family );
+	newstrs_free( &part, &family, NULL );
 }
 
 
@@ -254,9 +246,9 @@ output_names( fields *info, FILE *outptr, int level )
 static int
 output_finddateissued( fields *info, int level, int pos[3] )
 {
-	char      *src_names[] = { "YEAR", "MONTH", "DAY" };
-	char      *alt_names[] = { "PARTYEAR", "PARTMONTH", "PARTDAY" };
-	int       i, found = -1, ntypes = 3;
+	char      *src_names[] = { "YEAR", "MONTH", "DAY", "DATE" };
+	char      *alt_names[] = { "PARTYEAR", "PARTMONTH", "PARTDAY", "PARTDATE" };
+	int       i, found = -1, ntypes = 4;
 
 	for ( i=0; i<ntypes; ++i ) {
 		pos[i] = fields_find( info, src_names[i], level );
@@ -279,10 +271,9 @@ output_finddateissued( fields *info, int level, int pos[3] )
 }
 
 static void
-output_dateissued( fields *info, FILE *outptr, int level, int pos[3] )
+output_datepieces( fields *info, FILE *outptr, int pos[4] )
 {
 	int nprinted = 0, i;
-	output_tab1( outptr, increment_level(level,1), "<dateIssued>" );
 	for ( i=0; i<3 && pos[i]!=-1; ++i ) {
 		if ( nprinted>0 ) fprintf( outptr, "-" );
 		if ( i>0 && info->data[pos[i]].len==1 )
@@ -291,6 +282,24 @@ output_dateissued( fields *info, FILE *outptr, int level, int pos[3] )
 		nprinted++;
 		fields_setused( info, pos[i] );
 	}
+}
+
+static void
+output_dateall( fields *info, FILE *outptr, int pos )
+{
+	fprintf( outptr, "%s", info->data[pos].data );
+	fields_setused( info, pos );
+}
+
+static void
+output_dateissued( fields *info, FILE *outptr, int level, int pos[4] )
+{
+	output_tab1( outptr, increment_level(level,1), "<dateIssued>" );
+	if ( pos[0]!=-1 || pos[1]!=-1 || pos[2]!=-1 ) {
+		output_datepieces( info, outptr, pos );
+	} else {
+		output_dateall( info, outptr, pos[3] );
+	}
 	fprintf( outptr, "</dateIssued>\n" );
 }
 
@@ -304,7 +313,7 @@ output_origin( fields *info, FILE *outptr, int level )
 		{ "edition",	"EDITION",	0 }
 	};
 	int n, ntypes = sizeof( origin ) / sizeof ( convert );
-	int found, datefound, pos[5], date[3];
+	int found, datefound, pos[5], date[4];
 
 	/* find all information to be outputted */
 	found = -1;
@@ -359,55 +368,6 @@ output_toc( fields *info, FILE *outptr, int level )
 	output_fill2( outptr, level, "tableOfContents", info, n, 1 );
 }
 
-/* part date output
- *
- * <date>xxxx-xx-xx</date>
- *
- */
-static void
-output_partdate( fields *info, FILE *outptr, int level, int *wrote_header )
-{
-	convert parts[3] = {
-		{ "",	"PARTYEAR",                -1 },
-		{ "",	"PARTMONTH",               -1 },
-		{ "",	"PARTDAY",                 -1 },
-	};
-	int i, found = 0;
-	for ( i=0; i<3; ++i ) {
-		parts[i].code = fields_find( info, parts[i].internal, level );
-		found += ( parts[i].code!=-1 );
-	}
-	if ( !found ) return;
-
-	if ( !*wrote_header ) {
-		output_tab1( outptr, level, "<part>\n" );
-		*wrote_header = 1;
-	}
-
-	output_tab1( outptr, increment_level(level,1), "<date>" );
-
-	if ( parts[0].code!=-1 ) {
-		fprintf( outptr, "%s", info->data[ parts[0].code ].data);
-		fields_setused( info, parts[0].code );
-	}
-
-	if ( parts[1].code!=-1 ) {
-		if ( parts[0].code!=-1 ) fprintf( outptr, "-" );
-		else fprintf( outptr, "XXXX-" );
-		fprintf( outptr, "%s", info->data[parts[1].code].data );
-		fields_setused( info, parts[1].code );
-	}
-
-	if ( parts[2].code!=-1 ) {
-		if ( parts[1].code!=-1 ) fprintf( outptr, "-" );
-		else if ( parts[0].code!=-1 ) fprintf( outptr, "-XX-" );
-		fprintf( outptr, "%s", info->data[parts[2].code].data );
-		fields_setused( info, parts[2].code );
-	}
-
-	fprintf( outptr,"</date>\n");
-}
-
 /* detail output
  *
  * for example:
@@ -419,9 +379,9 @@ mods_output_detail( fields *info, FILE *outptr, int item, char *item_name,
 		int level )
 {
 	if ( item==-1 ) return;
-	output_tab0( outptr, increment_level(level,1) );
-	fprintf( outptr, "<detail type=\"%s\"><number>", item_name );
-	fprintf( outptr, "%s</number></detail>\n", info->data[item].data );
+	output_tab0( outptr, increment_level( level, 1 ) );
+	fprintf( outptr, "<detail type=\"%s\"><number>%s</number></detail>\n", 
+			item_name, info->data[item].data );
 	fields_setused( info, item );
 }
 
@@ -446,23 +406,73 @@ mods_output_extents( fields *info, FILE *outptr, int start, int end,
 }
 
 static void
-mods_output_partpages( fields *info, FILE *outptr, int level, int *wrote_header )
+try_output_partheader( FILE *outptr, int wrote_header, int level )
+{
+	if ( !wrote_header ) output_tab1( outptr, level, "<part>\n" );
+}
+
+static void
+try_output_partfooter( FILE *outptr, int wrote_header, int level )
+{
+	if ( wrote_header ) output_tab1( outptr, level, "</part>\n" );
+}
+
+/* part date output
+ *
+ * <date>xxxx-xx-xx</date>
+ *
+ */
+static int
+output_partdate( fields *info, FILE *outptr, int level, int wrote_header )
+{
+	convert parts[3] = {
+		{ "",	"PARTYEAR",                -1 },
+		{ "",	"PARTMONTH",               -1 },
+		{ "",	"PARTDAY",                 -1 },
+	};
+	int nparts = sizeof(parts)/sizeof(parts[0]);
+
+	if ( !find_alltags( info, parts, nparts, level ) ) return 0;
+
+	try_output_partheader( outptr, wrote_header, level );
+	output_tab1( outptr, increment_level(level,1), "<date>" );
+
+	if ( parts[0].code!=-1 ) {
+		fprintf( outptr, "%s", info->data[ parts[0].code ].data);
+		fields_setused( info, parts[0].code );
+	} else fprintf( outptr, "XXXX" );
+
+	if ( parts[1].code!=-1 ) {
+		fprintf( outptr, "-%s", info->data[parts[1].code].data );
+		fields_setused( info, parts[1].code );
+	}
+
+	if ( parts[2].code!=-1 ) {
+		if ( parts[1].code!=-1 ) fprintf( outptr, "-" );
+		else fprintf( outptr, "-XX-" );
+		fprintf( outptr, "%s", info->data[parts[2].code].data );
+		fields_setused( info, parts[2].code );
+	}
+
+	fprintf( outptr,"</date>\n");
+
+	return 1;
+}
+
+static int
+output_partpages( fields *info, FILE *outptr, int level, int wrote_header )
 {
 	convert parts[3] = {
 		{ "",  "PAGESTART",                -1 },
 		{ "",  "PAGEEND",                  -1 },
 		{ "",  "TOTALPAGES",               -1 }
 	};
-	int i, found = 0;
-	for ( i=0; i<3; ++i ) {
-		parts[i].code = fields_find( info, parts[i].internal, level );
-		found += ( parts[i].code!=-1 );
-	}
-	if ( !found ) return;
-	if ( !*wrote_header ) {
-		output_tab1( outptr, level, "<part>\n" );
-		*wrote_header = 1;
-	}
+	int nparts = sizeof(parts)/sizeof(parts[0]);
+
+	if ( !find_alltags( info, parts, nparts, level ) ) return 0;
+
+	try_output_partheader( outptr, wrote_header, level );
+
 	/* If PAGESTART or PAGEEND are  undefined */
 	if ( parts[0].code==-1 || parts[1].code==-1 ) {
 		if ( parts[0].code!=-1 )
@@ -480,10 +490,12 @@ mods_output_partpages( fields *info, FILE *outptr, int level, int *wrote_header
 		mods_output_extents( info, outptr, parts[0].code, 
 			parts[1].code, parts[2].code, "page", level ); 
 	}
+
+	return 1;
 }
 
-static void
-output_partelement( fields *info, FILE *outptr, int level, int *wrote_header )
+static int
+output_partelement( fields *info, FILE *outptr, int level, int wrote_header )
 {
 	convert parts[] = {
 		{ "volume",          "VOLUME",          -1 },
@@ -494,32 +506,28 @@ output_partelement( fields *info, FILE *outptr, int level, int *wrote_header )
 		{ "session",         "SESSION",         -1 },
 		{ "articlenumber",   "ARTICLENUMBER",   -1 }
 	};
-	int i, nparts = sizeof( parts ) / sizeof( convert ), found = 0;
-	for ( i=0; i<nparts; ++i ) {
-		parts[i].code = fields_find( info, parts[i].internal, level );
-		found += ( parts[i].code!=-1 );
-	}
-	if ( !found ) return;
-	if ( !(*wrote_header) ) {
-		output_tab1( outptr, level, "<part>\n" );
-		*wrote_header = 1;
-	}
+	int i, nparts = sizeof( parts ) / sizeof( convert );
+
+	if ( !find_alltags( info, parts, nparts, level ) ) return 0;
+	try_output_partheader( outptr, wrote_header, level );
+
 	for ( i=0; i<nparts; ++i ) {
 		if ( parts[i].code==-1 ) continue;
-		mods_output_detail( info, outptr, parts[i].code, 
-			parts[i].mods, level );
+		mods_output_detail( info, outptr, parts[i].code, parts[i].mods,
+				level );
 	}
+
+	return 1;
 }
 
 static void
 output_part( fields *info, FILE *outptr, int level )
 {
-	int wrote_header = 0;
-	output_partdate( info, outptr, level, &wrote_header );
-	output_partelement( info, outptr, level, &wrote_header );
-	mods_output_partpages( info, outptr, level, &wrote_header );
-	if ( wrote_header )
-		output_tab1( outptr, level, "</part>\n" );
+	int wrote_hdr;
+	wrote_hdr  = output_partdate( info, outptr, level, 0 );
+	wrote_hdr += output_partelement( info, outptr, level, wrote_hdr );
+	wrote_hdr += output_partpages( info, outptr, level, wrote_hdr );
+	try_output_partfooter( outptr, wrote_hdr, level );
 }
 
 static void
@@ -620,22 +628,31 @@ output_key( fields *info, FILE *outptr, int level )
 static void
 output_sn( fields *info, FILE *outptr, int level )
 {
-	char      *internal_names[] = { "ISBN", "LCCN", "ISSN", "REFNUM", 
-		"DOI" , "PUBMED", "MEDLINE", "PII", "ISIREFNUM", "ACCESSNUM" };
-	char      *mods_types[] = { "isbn", "lccn", "issn", "citekey", "doi",
-		"pubmed", "medline", "pii", "isi", "accessnum" };
-	int       n, ntypes = sizeof( internal_names ) / sizeof( char* );
+	convert sn_types[] = {
+		{ "isbn",      "ISBN",      0 },
+		{ "lccn",      "LCCN",      0 },
+		{ "issn",      "ISSN",      0 },
+		{ "citekey",   "REFNUM",    0 },
+		{ "doi",       "DOI",       0 },
+		{ "pubmed",    "PMID",      0 },
+		{ "medline",   "MEDLINE",   0 },
+		{ "pii",       "PII",       0 },
+		{ "arXiv",     "ARXIV",     0 },
+		{ "isi",       "ISIREFNUM", 0 },
+		{ "accessnum", "ACCESSNUM", 0 },
+	};
+	int n, ntypes = sizeof( sn_types ) / sizeof( sn_types[0] );
 	int       found, i;
 
 	found = fields_find ( info, "CALLNUMBER", level );
 	output_fill2( outptr, level, "classification", info, found, 1 );
 
 	for ( n=0; n<ntypes; ++n ) {
-		found = fields_find( info, internal_names[n], level );
+		found = fields_find( info, sn_types[n].internal, level );
 		if ( found==-1 ) continue;
 		output_tab0( outptr, level );
 		fprintf( outptr, "<identifier type=\"%s\">%s</identifier>\n",
-				mods_types[n],
+				sn_types[n].mods,
 				info->data[found].data
 		       );
 		fields_setused( info, found );
@@ -656,9 +673,10 @@ output_url( fields *info, FILE *outptr, int level )
 {
 	int location = fields_find( info, "LOCATION", level );
 	int url = fields_find( info, "URL", level );
+	int fileattach = fields_find( info, "FILEATTACH", level );
 	int pdflink = fields_find( info, "PDFLINK", level );
 	int i;
-	if ( url==-1 && location==-1 && pdflink==-1 ) return;
+	if ( url==-1 && location==-1 && pdflink==-1 && fileattach==-1) return;
 	output_tab1( outptr, level, "<location>\n" );
 	for ( i=0; i<info->nfields; ++i ) {
 		if ( info->level[i]!=level ) continue;
@@ -667,6 +685,11 @@ output_url( fields *info, FILE *outptr, int level )
 		} else if ( !strcasecmp( info->tag[i].data, "PDFLINK" ) ) {
 			output_fill2( outptr, increment_level(level,1), "url",
 				/*"urlType", "pdf",*/ info, i, 1 );
+		} else if ( !strcasecmp( info->tag[i].data, "FILEATTACH" ) ){
+			output_tab0( outptr, increment_level(level,1) );
+			fprintf( outptr, "<url displayLabel=\"Electronic full text\" access=\"raw object\">" );
+			fprintf( outptr, " %s</url>\n", info->data[i].data );
+			fields_setused( info, i );
 		}
 	}
 	if ( location!=-1 )
